#include "core.h"
#include "fmodsoundinstance.h"
#include "isound.h"
#include "iaudiosystem.h"
#include "configmanager.h"
#include "mathcore.h"
#include "sound3dlistener.h"

FMODSoundInstance::FMODSoundInstance( ISound* const pSound )
:	m_Channel( NULL )
{
	SetSound( pSound );
}

FMODSoundInstance::~FMODSoundInstance()
{
	if( !IsFinished() )
	{
		Stop();
	}

	// Delete the corresponding sound iff it's a stream (incidentally, stream sounds
	// only get automatically deleted if they're ever actually played).
	if( GetSound()->GetIsStream() )
	{
		SafeDelete( m_Sound );
	}

	m_Channel = NULL;
}

void FMODSoundInstance::Stop()
{
	FMOD_Channel_Stop( m_Channel );
}

void FMODSoundInstance::SetPriority( ESoundPriority Priority ) const
{
	if( Priority == ESP_Default )
	{
		FMOD_Channel_SetPriority( m_Channel, 128 );
	}
	else if( Priority == ESP_High )
	{
		FMOD_Channel_SetPriority( m_Channel, 0 );
	}
}

void FMODSoundInstance::SetPaused( bool Paused )
{
	FMOD_Channel_SetPaused( m_Channel, Paused );
}

void FMODSoundInstance::SetVolume( float Volume )
{
	FMOD_Channel_SetVolume( m_Channel, Volume );
}

void FMODSoundInstance::SetPan( float Pan )
{
	FMOD_Channel_SetPan( m_Channel, Pan );
}

/*virtual*/ bool FMODSoundInstance::IsPlaying() const
{
	FMOD_BOOL Playing = 0;
	FMOD_Channel_IsPlaying( m_Channel, &Playing );

	return Playing != 0;
}

bool FMODSoundInstance::IsFinished() const
{
	FMOD_BOOL Paused = 0;
	FMOD_BOOL Playing = 0;

	FMOD_Channel_GetPaused( m_Channel, &Paused );
	FMOD_Channel_IsPlaying( m_Channel, &Playing );

	return !( Paused != 0 || Playing != 0 );
}

float FMODSoundInstance::GetTimeElapsed() const
{
	uint Milliseconds = 0;
	FMOD_Channel_GetPosition( m_Channel, &Milliseconds, FMOD_TIMEUNIT_MS );
	return (float)Milliseconds / 1000.0f;
}
