#ifndef MODMANAGER_H
#define MODMANAGER_H

class ModDescription
{
public:
	ModDescription( SimpleString Name, SimpleString Path )
	:	m_Name( Name )
	,	m_Version( "" )
	,	m_WebsiteURL( "" )
	,	m_DownloadURL( "" )
	,	m_Path( Path )
	,	m_Space( Path )
	{}

	SimpleString m_Name;
	SimpleString m_Version;
	SimpleString m_WebsiteURL;
	SimpleString m_DownloadURL;
	SimpleString m_Path;
	HashedString m_Space;
};

class ModManager
{
public:
	static ModManager*		GetInstance();
	static void				DeleteInstance();

	bool InitializeMods( const char* ModCfgName );
	bool AddMod( const char* ModPath );

	// load one config file:
	//	- first try the packed config file from the packstream
	//	- then the unpacked config file rom the packstream
	//	- then the packed config file from thje directory
	//	- then the unpacked config file from the directory.
	bool LoadConfigFile( const char* ConfigFile );

	// open a packstream:
	// iterate backwards through m_Mods until we find the file. We load
	// packed after unpacked. If the file is not found in a mod, try the
	// default dir
	PackStream OpenPackStream( const char* FileName );

	static PackStream OpenStream( const char* FileName )
	{
		return GetInstance()->OpenPackStream( FileName );
	}

private:
	ModManager(); // load mods.cfg
	~ModManager();

	static ModManager* m_Instance;

	Array< ModDescription > m_Mods;
};

#endif /* MODMANAGER_H */
